from xlsxwriter.worksheet import Worksheet

class Workbook:
    def __init__(self, filename: str) -> None: ...
    def add_worksheet(self, title: str) -> Worksheet: ...
    def close(self) -> None: ...



"""
Algorithms for preprocessing the data.
"""

from typing import Optional

from xlsxwriter import Workbook
from xlsxwriter.worksheet import Worksheet

from bibx import Collection

from .sap import BRANCH, LEAF, ROOT, TRUNK, Sap


class Preprocess:
    def __init__(self, wos: Collection, scopus: Collection) -> None:
        self.wos = wos
        self.scopus = scopus
        self._merged: Optional[Collection] = None

    @property
    def merged(self):
        if self._merged is None:
            self._merged = self.wos.merge(self.scopus)
        return self._merged

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(wos={self.wos}, scopus={self.scopus})"

    def _write_collection_to_workseet(
        self,
        collection: Collection,
        workseet: Worksheet,
    ) -> None:
        for i, title in enumerate(
            [
                "Authors",
                "Year",
                "Title",
                "Journal",
                "Volume",
                "Issue",
                "Page",
                "DOI",
                "Times Cited",
                "Label",
            ]
        ):
            workseet.write(0, i, title)

        for i, row in enumerate(collection.articles, start=1):
            workseet.write(i, 0, "; ".join(row.authors))
            workseet.write(i, 1, row.year)
            workseet.write(i, 2, row.title)
            workseet.write(i, 3, row.journal)
            workseet.write(i, 4, row.volume)
            workseet.write(i, 5, row.issue)
            workseet.write(i, 6, row.page)
            workseet.write(i, 7, row.doi)
            workseet.write(i, 8, row.times_cited)
            workseet.write(i, 9, row.label)

    def write_merged_information(self, workseet: Worksheet) -> None:
        self._write_collection_to_workseet(self.merged, workseet)

    def write_wos_information(self, workseet: Worksheet) -> None:
        self._write_collection_to_workseet(self.wos, workseet)

    def write_scopus_information(self, workseet: Worksheet) -> None:
        self._write_collection_to_workseet(self.scopus, workseet)

    def write_reference_information(self, workseet: Worksheet) -> None:
        for i, title in enumerate(
            [
                "SR",
                "SR_ref",
                "Title",
                "Authors",
                "Journal",
                "Year",
                "Label",
            ]
        ):
            workseet.write(0, i, title)
        row = 1
        for article in self.merged.articles:
            for ref in article.references:
                workseet.write(row, 0, article.label)
                workseet.write(row, 1, ref.label)
                workseet.write(row, 2, ref.title)
                workseet.write(row, 3, "; ".join(ref.authors))
                workseet.write(row, 4, ref.journal)
                workseet.write(row, 5, ref.year)
                workseet.write(row, 6, ref.label)
                row += 1

    def write_journal_information(self, workseet: Worksheet) -> None:
        for i, title in enumerate(
            [
                "Label",
                "Journal",
                "Year",
                "Cited Journal",
                "Cited Year",
            ]
        ):
            workseet.write(0, i, title)
        row = 1
        for article in self.merged.articles:
            for ref in article.references:
                workseet.write(row, 0, article.label)
                workseet.write(row, 1, article.journal)
                workseet.write(row, 2, article.year)
                workseet.write(row, 3, ref.journal)
                workseet.write(row, 4, ref.year)
                row += 1

    def write_author_information(self, workseet: Worksheet) -> None:
        for i, title in enumerate(
            [
                "Author",
                "Year",
                "Cited Author",
                "Cited Year",
            ]
        ):
            workseet.write(0, i, title)
        row = 1
        for article in self.merged.articles:
            for ref in article.references:
                for author in article.authors:
                    for cited_author in ref.authors:
                        workseet.write(row, 0, author)
                        workseet.write(row, 1, article.year)
                        workseet.write(row, 2, cited_author)
                        workseet.write(row, 3, ref.year)
                        row += 1

    def write_times_cited_information(self, workseet: Worksheet) -> None:
        for i, title in enumerate(
            [
                "Year",
                "Times Cited Sum",
                "Times Cited Percentage",
            ]
        ):
            workseet.write(0, i, title)
        years = self.merged.cited_by_year()
        total = sum(years.values())
        for i, (year, times_cited) in enumerate(years.items(), start=1):
            workseet.write(i, 0, year)
            workseet.write(i, 1, times_cited)
            workseet.write(i, 2, times_cited / total)

    @staticmethod
    def _get_tos(data: dict) -> str:
        if data[ROOT] > 0:
            return "Root"
        elif data[TRUNK] > 0:
            return "Trunk"
        elif data[LEAF] > 0:
            return "Leaf"
        elif data[BRANCH] > 0:
            return f"Branch {data[BRANCH]}"
        return "_"

    def write_tree_of_science_information(self, workseet: Worksheet) -> None:
        s = Sap()
        g = s.create_graph(self.merged)
        g = s.clean_graph(g)
        g = s.tree(g)

        for i, title in enumerate(
            [
                "TOS",
                "Label",
                "Authors",
                "Year",
                "Title",
                "Journal",
                "Volume",
                "Issue",
                "Page",
                "DOI",
                "Times Cited",
            ]
        ):
            workseet.write(0, i, title)

        i = 1
        for label, data in sorted(
            g.nodes(data=True), key=lambda x: self._get_tos(x[1])
        ):
            tos = self._get_tos(data)
            if tos == "_":
                continue
            workseet.write(i, 0, tos)
            workseet.write(i, 1, label)
            workseet.write(i, 2, "; ".join(data["authors"]))
            workseet.write(i, 3, data["year"])
            workseet.write(i, 4, data["title"])
            workseet.write(i, 5, data["journal"])
            workseet.write(i, 6, data["volume"])
            workseet.write(i, 7, data["issue"])
            workseet.write(i, 8, data["page"])
            workseet.write(i, 9, data["doi"])
            workseet.write(i, 10, data["times_cited"])
            i += 1

    def create_workbook(self, filename: str) -> None:
        workbook = Workbook(filename)
        self.write_merged_information(workbook.add_worksheet("Merged"))
        self.write_wos_information(workbook.add_worksheet("WOS"))
        self.write_scopus_information(workbook.add_worksheet("Scopus"))
        self.write_reference_information(workbook.add_worksheet("References"))
        self.write_journal_information(workbook.add_worksheet("Journals"))
        self.write_author_information(workbook.add_worksheet("Authors"))
        self.write_times_cited_information(workbook.add_worksheet("Times Cited"))
        self.write_tree_of_science_information(
            workbook.add_worksheet("Tree of Science")
        )
        workbook.close()



import io

from bibx import read_scopus_ris


def test_scopus_works():
    file = io.StringIO(
        """
TY  - JOUR
TI  - FORC signatures and switching-field distributions of dipolar coupled nanowire-based hysterons
T2  - Journal of Applied Physics
J2  - J Appl Phys
VL  - 128
IS  - 9
PY  - 2020
DO  - 10.1063/5.0020407
AU  - Pierrot, A.
AU  - Béron, F.
AU  - Blon, T.
N1  - Export Date: 15 October 2020
M3  - Article
DB  - Scopus
C7  - 093903
N1  - References: Mayergoyz, I.D., (1986) Phys. Rev. Lett., 56, p. 1518;
Preisach, F., (1935) Z. Phys., 94, p. 277;
Roberts, A.P., Heslop, D., Zhao, X., Pike, C.R., (2014) Rev. Geophys., 52, pp. 557-602. , https://doi.org/10.1002/2014RG000462;
Pike, C.R., Roberts, A.P., Verosub, K.L., (1999) J. Appl. Phys., 85, p. 6660;
Dobrot, C.-I., Stancu, A., (2013) J. Appl. Phys., 113;
Carvallo, C., Muxworthy, A., (2006) Geochem. Geophys. Geosyst., 7, p. Q09003. , https://doi.org/10.1029/2006GC001299;
Lappe, S.-C.L.L., Feinberg, J.M., Muxworthy, A., Harrison, R.J., (2013) Geochem. Geophys. Geosyst., 14, p. 2143. , https://doi.org/10.1002/ggge.20141;
Zhao, X., Roberts, A.P., Heslop, D., Paterson, G.A., Li, Y., Li, J., (2017) J. Geophys. Res. Solid Earth, 122, p. 4767. , https://doi.org/10.1002/2016JB013683;
Papusoi, C., Srinivasan, K., Acharya, R., (2011) J. Appl. Phys., 110;
Papusoi, C., Desai, M., Acharya, R., (2015) J. Phys. D Appl. Phys., 48;
UR  - https://www.scopus.com/inward/record.uri?eid=2-s2.0-85091557653&doi=10.1063%2f5.0020407&partnerID=40&md5=1471f5e876fae65e040690b345036add
ER  -
"""
    )
    data = read_scopus_ris(file)
    assert len(data.articles) == 1
    (article,) = data.articles
    assert (
        article.title
        == "FORC signatures and switching-field distributions of dipolar coupled nanowire-based hysterons"
    )
    assert article.year == 2020
    assert len(list(data.citation_pairs)) == 10
    assert article.times_cited is None



from bibx._entities.article import Article
from bibx._entities.collection import Collection

articles = [
    Article(
        authors=["A"],
        year=2010,
        title="Aa",
        journal="Aaa",
        volume="1",
        page="10",
        doi="1",
        times_cited=0,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["B"],
        year=2000,
        title="Bb",
        journal="Abb",
        volume="2",
        page="11",
        doi="12",
        times_cited=2,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["C"],
        year=2021,
        title="Cc",
        journal="Acc",
        volume="3",
        page="12",
        doi="13",
        times_cited=12,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["D"],
        year=2022,
        title="Dd",
        journal="Add",
        volume="4",
        page="13",
        doi="14",
        times_cited=2,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["E"],
        year=2005,
        title="Ee",
        journal="Aee",
        volume="5",
        page="14",
        doi="15",
        times_cited=0,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["F"],
        year=2005,
        title="Ff",
        journal="Aff",
        volume="6",
        page="15",
        doi="16",
        times_cited=None,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["J"],
        year=2010,
        title="Jj",
        journal="Ajj",
        volume="7",
        page="16",
        doi="17",
        times_cited=1,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["H"],
        year=2000,
        title="Hh",
        journal="Ahh",
        volume="8",
        page="17",
        doi="18",
        times_cited=20,
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["I"],
        year=2021,
        title="Ii",
        journal="Aii",
        volume="9",
        page="18",
        doi="19",
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["I"],
        year=None,
        title="Ii",
        journal="Aii",
        volume="9",
        page="18",
        doi="19",
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
    Article(
        authors=["I"],
        title="Ii",
        journal="Aii",
        volume="9",
        page="18",
        doi="19",
        references=[],
        keywords=[],
        sources=set(),
        extra={},
        _label=None,
    ),
]


def test_published_by_year():
    collection = Collection(articles=articles)

    res = collection.published_by_year()
    assert res.get(2000) == 2
    assert res.get(2001) == 0
    assert res.get(2002) == 0
    assert res.get(2005) == 2
    assert res.get(2010) == 2
    assert res.get(2021) == 2
    assert res.get(2022) == 1
    assert res.get(2023) == 0


def test_cited_by_year():
    collection = Collection(articles=articles)

    res = collection.cited_by_year()
    assert res.get(2000) == 22
    assert res.get(2001) == 0
    assert res.get(2002) == 0
    assert res.get(2005) == 0
    assert res.get(2010) == 1
    assert res.get(2021) == 12
    assert res.get(2022) == 2
    assert res.get(2023) == 0



from typing import Any, Iterable, Iterator, List

from _typeshed import Self
from networkx.classes.reportviews import NodeView

__version__: str

class Graph:
    nodes: NodeView
    def subgraph(self: Self, nodes: Iterable) -> Self: ...
    def copy(self: Self) -> Self: ...
    def add_node(self, node: str, **kwargs) -> None: ...
    def add_edge(self, u: str, v: str, **kwargs) -> None: ...
    def add_edges_from(self, edges: Iterable) -> None: ...
    def remove_nodes_from(self, edges: Iterable) -> None: ...
    def remove_edges_from(self, edges: Iterable) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def in_degree(self, node: str) -> int: ...
    def out_degree(self, node: str) -> int: ...

class DiGraph(Graph):
    def to_undirected(self) -> Graph: ...
    def successors(self, node: str) -> Iterable[str]: ...
    def predecessors(self, node: str) -> Iterable[str]: ...

# NOTE: The attr parameter of this function is quite more complicated than this
def set_node_attributes(g: DiGraph, attr: Any, name: str) -> None: ...
def topological_sort(g: DiGraph) -> Iterable[str]: ...
def selfloop_edges(g: DiGraph) -> Iterable[str]: ...
def weakly_connected_components(g: DiGraph) -> Iterable[List[str]]: ...
def strongly_connected_components(g: DiGraph) -> Iterable[List[str]]: ...



from typing import Union

class Worksheet:
    # NOTE: In reality, the write method has many overloads.
    def write(
        self,
        row: int,
        col: int,
        value: Union[int, float, str, None],
    ) -> None: ...



import logging
from typing import TextIO

from bibx._entities.article import Article
from bibx._entities.collection import Collection
from bibx._entities.collection_builders.scopus_bib import ScopusBibCollectionBuilder
from bibx._entities.collection_builders.scopus_ris import ScopusRisCollectionBuilder
from bibx._entities.collection_builders.wos import WosCollectionBuilder
from bibx.algorithms.sap import Sap
from bibx.exceptions import BibXError

logger = logging.getLogger(__name__)

__all__ = [
    "Article",
    "Collection",
    "Sap",
    "read_scopus_bib",
    "read_scopus_ris",
    "read_wos",
    "read_any",
]

__version__ = "0.1.1"


def read_scopus_bib(*files: TextIO) -> Collection:
    """
    Takes any number of bibtex files from scopus and generates a collection.

    :param files: Scopus bib files open.
    :return: the collection
    """
    return ScopusBibCollectionBuilder(*files).build()


def read_scopus_ris(*files: TextIO) -> Collection:
    """
    Takes any number of ris files from scopus and generates a collection.

    :param files: Scopus bib files open.
    :return: the collection
    """
    return ScopusRisCollectionBuilder(*files).build()


def read_wos(*files: TextIO) -> Collection:
    """
    Takes any number of wos text files and returns a collection.

    :param files: WoS files open.
    :return: the collection
    """
    return WosCollectionBuilder(*files).build()


def read_any(file: TextIO) -> Collection:
    """
    Tries to read a file with the supported formats.
    """
    for handler in (read_wos, read_scopus_ris, read_scopus_bib):
        try:
            return handler(file)
        except BibXError as e:
            logger.debug(f"Error: {e}")
        except ValueError:
            logger.debug(
                f"Error: the {handler.__name__} function does not support this file"
            )
    raise ValueError("Unsupported file type")



from dataclasses import dataclass, field
from typing import List, Mapping, Optional, Set


@dataclass
class Article:
    authors: List[str] = field(default_factory=list)
    year: Optional[int] = None
    title: Optional[str] = None
    journal: Optional[str] = None
    volume: Optional[str] = None
    issue: Optional[str] = None
    page: Optional[str] = None
    doi: Optional[str] = None
    _label: Optional[str] = None
    times_cited: Optional[int] = None
    references: List["Article"] = field(default_factory=list)
    keywords: List[str] = field(default_factory=list)
    sources: Set[str] = field(default_factory=set)
    extra: Mapping = field(default_factory=dict)

    @property
    def key(self):
        if self.authors:
            author = self.authors[0].split(" ")[0].replace(",", "")
        else:
            author = "anonymous"
        year = self.year
        return f"{author}{year}".lower()

    @property
    def label(self):
        if self._label is not None:
            return self._label
        pieces = {
            "AU": self.authors[0].replace(",", "") if self.authors else "anonymous",
            "PY": str(self.year) if self.year else None,
            "J9": str(self.journal) if self.journal else None,
            "VL": f"V{self.volume}" if self.volume else None,
            "BP": f"P{self.page}" if self.page else None,
            "DI": f"DOI {self.doi}" if self.doi else None,
        }
        return ", ".join(value for value in pieces.values() if value)



import io

from bibx import read_wos


def test_scopus_works():
    file = io.StringIO(
        """
FN Thomson Reuters Web of Science™
VR 1.0
PT J
AU Sun, ZW
   Russell, TP
AF Sun, Zhiwei
   Russell, Thomas P.
TI In situ grazing incidence small-angle X-ray scattering study of solvent
   vapor annealing in lamellae-forming block copolymer thin films:
   Trade-off of defects in deswelling
SO JOURNAL OF POLYMER SCIENCE PART B-POLYMER PHYSICS
LA English
DT Article
DE annealing; block copolymers; self-assembly; thin films; X-ray
ID BIT-PATTERNED MEDIA; LITHOGRAPHY; GRAPHENE; ARRAYS; ORIENTATION;
   NANOWIRES; PARALLEL; BEHAVIOR; INPLANE; DENSITY
AB Solvent vapor annealing (SVA) is one route to prepare block copolymer (BCP) thin films with long-range lateral ordering. The lattice defects in the spin-coated BCP thin film can be effectively and rapidly reduced using SVA. The solvent evaporation after annealing was shown to have a significant impact on the in-plane ordering of BCP microdomains. However, the effect of solvent evaporation on the out-of-plane defects in BCPs has not been considered. Using grazing-incidence x-ray scattering, the morphology evolution of lamellae-forming poly(2-vinlypyridine)-b-polystyrene-b-poly(2vinylpyridine) triblock copolymers, having lamellar microdomains oriented normal to substrate surface during SVA, was studied in this work. A micelle to lamellae transformation was observed during solvent uptake. The influence of solvent swelling ratio and solvent removal rate on both the in-plane and out-of-plane defect density was studied. It shows that there is a trade-off between the in-plane and out-of-plane defect densities during solvent evaporation. (c) 2017 Wiley Periodicals, Inc. J. Polym. Sci., Part B: Polym. Phys. 2017, 55, 980-989
C1 [Sun, Zhiwei; Russell, Thomas P.] Univ Massachusetts Amherst, Dept Polymer Sci & Engn, Amherst, MA 01003 USA.
   [Russell, Thomas P.] Lawrence Berkeley Natl Lab, Div Mat Sci, Berkeley, CA 94720 USA.
   [Russell, Thomas P.] Beijing Univ Chem Technol, Beijing Adv Innovat Ctr Soft Matter Sci & Engn, Beijing, Peoples R China.
RP Russell, TP (reprint author), Univ Massachusetts Amherst, Dept Polymer Sci & Engn, Amherst, MA 01003 USA.; Russell, TP (reprint author), Lawrence Berkeley Natl Lab, Div Mat Sci, Berkeley, CA 94720 USA.; Russell, TP (reprint author), Beijing Univ Chem Technol, Beijing Adv Innovat Ctr Soft Matter Sci & Engn, Beijing, Peoples R China.
EM russell@mail.pse.umass.edu
FU U.S. Department of Energy BES [BES-DE-FG02-96ER45612]; Director of the
   Office of Science, Office of Basic Energy Sciences, of the U.S.
   Department of Energy [DE-AC02-05CH11231]; Office of Science, Office of
   Basic Energy Sciences, of the U.S. Department of Energy
   [DE-AC02-05CH11231]
FX The authors acknowledge the facility support in Advanced Light Source
   and Molecular Foundry in Lawrence Berkeley National Laboratory. This
   work was supported by the U.S. Department of Energy BES under contract
   BES-DE-FG02-96ER45612. The GISAXS characterization in beamline 7.3.3 of
   the Advanced Light Source is supported by the Director of the Office of
   Science, Office of Basic Energy Sciences, of the U.S. Department of
   Energy under contract no. DE-AC02-05CH11231. The SEM and AFM
   characterization in the Molecular Foundry was supported by the Office of
   Science, Office of Basic Energy Sciences, of the U.S. Department of
   Energy under contract no. DE-AC02-05CH11231.
CR Bai W, 2015, MACROMOLECULES, V48, P8574, DOI 10.1021/acs.macromol.5b02174
   Bosworth JK, 2011, MACROMOLECULES, V44, P9196, DOI 10.1021/ma201967a
   Bosworth JK, 2010, J PHOTOPOLYM SCI TEC, V23, P145, DOI 10.2494/photopolymer.23.145
   Chai J, 2008, ACS NANO, V2, P489, DOI 10.1021/nn700341s
   Chai J, 2007, NAT NANOTECHNOL, V2, P500, DOI 10.1038/nnano.2007.227
   Choi S, 2012, SOFT MATTER, V8, P3463, DOI 10.1039/c2sm07297a
   Di ZY, 2012, MACROMOLECULES, V45, P5185, DOI 10.1021/ma3004136
   Farrell RA, 2012, NANOSCALE, V4, P3228, DOI 10.1039/c2nr00018k
   Gowd E. B., 2010, IOP C SER MAT SCI EN, V14
   Gu XD, 2014, ADV MATER, V26, P273, DOI 10.1002/adma.201302562
   Gunkel I, 2016, J POLYM SCI POL PHYS, V54, P331, DOI 10.1002/polb.23933
   Ilavsky J, 2012, J APPL CRYSTALLOGR, V45, P324, DOI 10.1107/S0021889812004037
   Jeong SJ, 2010, NANO LETT, V10, P3500, DOI 10.1021/nl101637f
   Ji S, 2008, MACROMOLECULES, V41, P9098, DOI 10.1021/ma801861h
   Khaira GS, 2014, ACS MACRO LETT, V3, P747, DOI 10.1021/mz5002349
   Kikitsu A, 2013, IEEE T MAGN, V49, P693, DOI 10.1109/TMAG.2012.2226566
   Kim BH, 2011, ADV MATER, V23, P5618, DOI 10.1002/adma.201103650
   Kim BH, 2010, ACS NANO, V4, P5464, DOI 10.1021/nn101491g
   Kurihara M, 2013, JPN J APPL PHYS, V52, DOI 10.7567/JJAP.52.086201
   Liu GX, 2012, ACS NANO, V6, P6786, DOI 10.1021/nn301515a
   Mahadevapuram N, 2016, J POLYM SCI POL PHYS, V54, P339, DOI 10.1002/polb.23937
   Paik MY, 2010, MACROMOLECULES, V43, P4253, DOI 10.1021/ma902646t
   Sinturel C, 2014, ACS APPL MATER INTER, V6, P12146, DOI 10.1021/am504086x
   Sun ZW, 2015, ADV MATER, V27, P4364, DOI 10.1002/adma.201501585
   Vu T, 2011, MACROMOLECULES, V44, P6121, DOI 10.1021/ma2009222
   Thurn-Albrecht T, 2000, SCIENCE, V290, P2126, DOI 10.1126/science.290.5499.2126
   Wan L., 2012, MOEMS, V11, P31405
   Wang JY, 2008, LANGMUIR, V24, P3545, DOI 10.1021/la703559q
   Xiao S., 2013, MOEMS, V12
   Xiao SG, 2014, ACS NANO, V8, P11854, DOI 10.1021/nn505630t
   Xiao SG, 2014, J POLYM SCI POL PHYS, V52, P361, DOI 10.1002/polb.23433
   Yamamoto R, 2014, IEEE T MAGN, V50, DOI 10.1109/TMAG.2013.2284474
   Yang X., 2014, MOEMS, V13
   Yang X., 2013, J MATER RES, V2013, P1
   Yang XM, 2014, NANOTECHNOLOGY, V25, DOI 10.1088/0957-4484/25/39/395301
   Yang XM, 2009, ACS NANO, V3, P1844, DOI 10.1021/nn900073r
   Zhang JQ, 2014, MACROMOLECULES, V47, P5711, DOI 10.1021/ma500633b
NR 37
TC 0
Z9 0
U1 1
U2 1
PU WILEY
PI HOBOKEN
PA 111 RIVER ST, HOBOKEN 07030-5774, NJ USA
SN 0887-6266
EI 1099-0488
J9 J POLYM SCI POL PHYS
JI J. Polym. Sci. Pt. B-Polym. Phys.
PD JUL 1
PY 2017
VL 55
IS 13
BP 980
EP 989
DI 10.1002/polb.24346
PG 10
WC Polymer Science
SC Polymer Science
GA EU7BQ
UT WOS:000401190100002
ER
"""
    )
    data = read_wos(file)
    assert len(data.articles) == 1
    (article,) = data.articles
    assert (
        article.title
        == "In situ grazing incidence small-angle X-ray scattering study of solvent vapor annealing in lamellae-forming block copolymer thin films: Trade-off of defects in deswelling"
    )
    assert len(list(data.citation_pairs)) == 37
    assert article.times_cited == 0



from collections.abc import Callable
from enum import Enum
from typing import List, TextIO

import networkx as nx
import typer
from rich import print

from bibx import Collection, read_any, read_scopus_bib, read_scopus_ris, read_wos
from bibx.algorithms.preprocess import Preprocess
from bibx.algorithms.sap import Sap

app = typer.Typer()


class Format(Enum):
    WOS = "wos"
    RIS = "ris"
    BIB = "bib"


@app.command()
def describe(format: Format, filename: str):
    """
    Parses a file and provides a short description.
    """
    if format == Format.WOS:
        c = read_wos(open(filename))
        print(":boom: the file satisfies the ISI WOS format")
        print(f"There are {len(c.articles)} records parsed")
    if format == Format.RIS:
        c = read_scopus_ris(open(filename))
        print(":boom: the file satisfies the ISI WOS format")
        print(f"There are {len(c.articles)} records parsed")
    if format == Format.BIB:
        c = read_scopus_bib(open(filename))
        print(":boom: the file satisfies the ISI WOS format")
        print(f"There are {len(c.articles)} records parsed")


@app.command()
def toy_sap():
    """
    Runs the sap algorithm on a toy graph.
    """
    graph = nx.DiGraph()
    for node in "abcde":
        graph.add_node(node, year=2000)
    graph.add_edge("a", "b")
    for node in "cde":
        graph.add_edge("b", node)
    s = Sap()
    graph = s.tree(graph)
    print(graph)


@app.command()
def sap(filename: str):
    """
    Runs the sap algorithm on a seed file of any supported format.
    """
    with open(filename) as f:
        collection = read_any(f)

    s = Sap()
    graph = s.create_graph(collection)
    graph = s.clean_graph(graph)
    graph = s.tree(graph)
    print(graph)


def _read_many(
    reader: Callable[[TextIO], Collection],
    *filenames: str,
) -> Collection:
    first, *rest = filenames
    with open(first) as f:
        collection = reader(f)
    for filename in rest:
        with open(filename) as f:
            collection = collection.merge(reader(f))
    return collection


@app.command()
def preprocess(
    output: str,
    wos: List[str] = typer.Option(help="WoS files to pre process"),
    scopus: List[str] = typer.Option(help="scopus files to preprocess"),
):
    """
    Preprocesses a collection.
    """
    wos_collection = _read_many(read_wos, *wos)
    scopus_collection = _read_many(read_scopus_ris, *scopus)
    p = Preprocess(wos_collection, scopus_collection)
    p.create_workbook(output)


if __name__ == "__main__":
    app()



import logging
import re
from collections import defaultdict
from typing import Dict, Iterable, List, Optional, TextIO, Tuple

from bibx._entities.article import Article
from bibx._entities.collection import Collection
from bibx._entities.collection_builders.base import CollectionBuilder
from bibx.exceptions import InvalidScopusFile, MissingCriticalInformation

logger = logging.getLogger(__name__)


def _size(file) -> int:
    file.seek(0, 2)
    size = file.tell()
    file.seek(0)
    return size


def _int_or_nothing(raw: Optional[List[str]]) -> Optional[int]:
    if not raw:
        return None
    try:
        return int(raw[0])
    except TypeError:
        return None


def _joined(raw: Optional[List[str]]) -> Optional[str]:
    if not raw:
        return None
    return " ".join(raw)


class ScopusRisCollectionBuilder(CollectionBuilder):
    def __init__(self, *ris_files: TextIO):
        self._files = ris_files
        for file in self._files:
            file.seek(0)

    def build(self) -> Collection:
        articles = self._get_articles_from_files()
        return Collection(list(articles))

    def _get_articles_from_files(self) -> Iterable[Article]:
        for file in self._files:
            yield from self._parse_file(file)

    @staticmethod
    def _find_volume_info(ref: str) -> Tuple[Dict[str, str], str]:
        volume_pattern = re.compile(r"(?P<volume>\d+)( \((?P<issue>.+?)\))?")
        page_pattern = re.compile(r"(pp?\. (?P<page>\w+)(-[^,\s]+)?)")
        page = page_pattern.search(ref)
        last_index = 0
        if page:
            last_index = page.lastindex or 0
            first, *_ = ref.split(page.group())
            volume = volume_pattern.search(first)
        else:
            volume = volume_pattern.search(ref)
            if volume:
                last_index = volume.lastindex or 0

        if not page and not volume:
            return {}, ref

        data = {}
        if page:
            data.update(page.groupdict())
        if volume:
            data.update(volume.groupdict())

        if "volume" in data and data["volume"]:
            data["volume"] = f"V{data['volume']}"
        if "page" in data and data["page"]:
            data["page"] = f"P{data['page']}"

        return data, ref[last_index:]

    @staticmethod
    def _find_doi(ref: str) -> Tuple[Optional[str], str]:
        pattern = re.compile(
            r"((doi.org\/)|(aps.org\/doi\/)|(DOI:?)) ?(?P<doi>[^\s,;:]{5,})", re.I
        )
        result = re.search(pattern, ref)
        if result is None or "doi" not in result.groupdict():
            return None, ref
        doi = result.groupdict()["doi"]
        return f"DOI {doi}", ref[result.lastindex :]

    @classmethod
    def _article_form_reference(cls, scopusref: str) -> Article:
        authors, year, rest = re.split(r"(\(\d{4}\))", scopusref, maxsplit=1)
        first_name, last_name, *_ = authors.split(", ")
        year = year[1:-1]
        journal, rest = rest.split(", ", 1)
        volume_info, rest = cls._find_volume_info(rest)
        doi, _ = cls._find_doi(scopusref)
        if not authors or not year:
            raise MissingCriticalInformation()
        return Article(
            authors=[f"{first_name} {last_name.replace(' ', '').replace('.', '')}"],
            year=int(year),
            journal=(
                journal.strip().replace(".", "").upper()
                if not journal.isspace()
                else None
            ),
            volume=volume_info.get("volume"),
            page=volume_info.get("page"),
            doi=doi,
        )

    @classmethod
    def _parse_references(cls, refs: List[str]) -> List[Article]:
        if not refs:
            return []
        result = []
        for ref in refs:
            try:
                result.append(cls._article_form_reference(ref))
            except (KeyError, IndexError, TypeError, ValueError):
                logging.debug(f"Ignoring invalid reference {ref}")
        return result

    @staticmethod
    def _ris_to_dict(record: str) -> Dict[str, List[str]]:
        RIS_PATTERN = re.compile(
            r"^(((?P<key>[A-Z0-9]{2}))[ ]{2}-[ ]{1})?(?P<value>(.*))$"
        )
        parsed = defaultdict(list)
        current = None

        for line in record.split("\n"):
            match = RIS_PATTERN.match(line)
            if not match:
                raise InvalidScopusFile()
            data = match.groupdict()
            key = data.get("key")
            value = data.get("value")
            if "ER" in data:
                break
            if key:
                if key == "N1" and value and ":" in value:
                    label, value = value.split(":", 1)
                    value = value.strip()
                    current = f"{key}:{label.strip()}"
                else:
                    current = data["key"]
            if value and current:
                parsed[current].append(data.get("value", ""))
        return dict(parsed)

    @classmethod
    def _article_from_record(cls, record: str) -> Article:
        data = cls._ris_to_dict(record)
        year = _int_or_nothing(data.get("PY", []))
        times_cited = _int_or_nothing(data.get("TC"))
        authors = data.get("AU", [])
        return Article(
            title=_joined(data.get("TI")),
            authors=authors,
            year=year,
            journal=_joined(data.get("J2")),
            volume=_joined(data.get("VL")),
            issue=_joined(data.get("IS")),
            page=_joined(data.get("SP")),
            doi=_joined(data.get("DO")),
            keywords=data.get("KW", []),
            references=cls._parse_references(data.get("N1:References", [])),
            sources={"scopus"},
            extra=data,
            times_cited=times_cited,
        )

    @classmethod
    def _parse_file(cls, file: TextIO) -> Iterable[Article]:
        if not _size(file):
            return []
        for item in file.read().split("\n\n"):
            if item.isspace():
                continue
            try:
                article = cls._article_from_record(item.strip())
                yield article
            except MissingCriticalInformation:
                logger.info("Missing critical information for record %s", item)



import datetime
import logging
from dataclasses import dataclass
from typing import Dict, Iterable, List, Tuple

from bibx._entities.article import Article

logger = logging.getLogger(__name__)


@dataclass
class Collection:
    articles: List[Article]

    def merge(self, other: "Collection") -> "Collection":
        """
        Creates a new collection merging the articles by key.

        :param other: collection to merge to.
        :return: a new collection object.
        """
        keys = {a.key for a in self.articles}
        merged = self.articles[:]
        merged.extend(a for a in other.articles if a.key not in keys)
        return Collection(merged)

    @property
    def all_articles(self) -> Iterable[Article]:
        """
        Yields all articles and references

        :return: an iterable over `Article`.
        """
        cache = {article.key: article for article in self.articles}
        for article in self.articles:
            for reference in article.references:
                cache.setdefault(reference.key, reference)
        yield from cache.values()

    @property
    def citation_pairs(self) -> Iterable[Tuple[Article, Article]]:
        cache = {article.key: article for article in self.articles}
        for article in self.articles:
            if not article.references:
                continue
            for reference in article.references:
                # Yield a full article if we have the metadata for a citation
                if article.key != reference.key and reference.key in cache:
                    logger.debug("Found a cache hit for key %s", reference.key)
                    yield article, cache[reference.key]
                else:
                    yield article, reference

    @property
    def _first_year(self) -> int:
        """
        Returns the year of the first article in the collection.

        :return: an integer.
        """
        return min(
            [article.year for article in self.articles if article.year is not None]
        )

    def published_by_year(self) -> Dict[int, int]:
        """
        Returns a dictionary where the key is the year of publication and the value is the number of articles
        published that year. The dictionary starts from the oldest article to the current year consecutively.
        If a year has no publications the value will be zero.

        :return: a dictionary with the number of articles published each year.
        """
        current_year = datetime.date.today().year
        years = {}
        for year in range(self._first_year, current_year + 1):
            years[year] = 0

        for article in self.articles:
            if article.year is None:
                continue
            if article.year in years:
                years[article.year] += 1
            else:
                years[article.year] = 1
        return years

    def cited_by_year(self) -> Dict[int, int]:
        """
        Returns a dictionary where the key is the year of publication and the value is the number of
        citations in that year. The dictionary starts from the oldest article to the current year consecutively.
        If a year has no citations the value will be zero.

        :return: a dictionary with the number of citations each year.
        """
        current_year = datetime.date.today().year
        cited_items_per_year = {}
        for year in range(self._first_year, current_year + 1):
            cited_items_per_year[year] = 0

        for article in self.articles:
            if article.times_cited is None or article.year is None:
                continue
            if article.year in cited_items_per_year:
                cited_items_per_year[article.year] += article.times_cited
            else:
                cited_items_per_year[article.year] = article.times_cited

        return cited_items_per_year
